library(car)
library(olsrr)
library(epiR)
source("mspe.R")


setwd() # set the working directory to the folder that the R script is saved in
load("Regression.Rda")


### Conduct some data exploration

pairs(dat) #note plotting and summarizing the entire dataset will only work with smaller datasets
cor(dat)
summary(dat)


#######################################################################
#######################################################################
## On your own:
## plot y vs. some of the predictor variables and their squares



######################################################################## 





# Let's fit the model with all variables
full.fit = lm(y ~ ., dat)

summary(full.fit) ## What is wrong with this model fit?



# Individual regressions will often get you the incorrect inference
summary(lm(y ~ x1, dat))
summary(lm(y ~ x2, dat))
summary(lm(y ~ x3, dat))
summary(lm(y ~ x4, dat))
summary(lm(y ~ x5, dat))
summary(lm(y ~ x6, dat))
summary(lm(y ~ x7, dat))
summary(lm(y ~ x8, dat))
summary(lm(y ~ x9, dat))
summary(lm(y ~ x10, dat))

### What happens if we conduct automated model selection?

# All possible models
all_fit = ols_step_all_possible(full.fit)


## Explore the object all_fit
names(all_fit)
head(all_fit)


### Find the model with the lowest AIC
all_fit[which.min(all_fit[, "aic"]), "predictors"]


#######################################################################
#######################################################################
## On your own:
## Find the model with the lowest Adjusted R-squared



########################################################################


### Conduct an automated forward selection based on the p-value 
### of the parameter, with a p-value cutoff of 0.2

# Forward Selection
ols_step_forward_p(full.fit, penter = 0.2)



#######################################################################
#######################################################################
### On Your own: 
### Conduct an automated Backward selection based on the p-value 
### of the parameter, with a p-value cutoff of 0.3


#######################################################################

### Conduct a backwards and forward stepwise regression with the p-value
# Stepwise Regression
ols_step_both_p(full.fit)


### Are all the models the same? How do you know which one to choose?



### Conduct an automated forward selection based on the aic
# Stepwise AIC Forward Regression
ols_step_forward_aic(full.fit)



#######################################################################
#######################################################################
### On your own:
### Conduct an automated forward selection based on the aic

#Stepwise AIC Backward Regression



#######################################################################

#### Are all the models the same? 
#### Based on information you gained in the the data exploration, 
#### what should you do to improve this model selection process?


### Let's explore the collinearity of the full model
vif(full.fit)

### Remember that we want models with a Variance Inflation Factor (VIF) less than 10

# Remove variable with highest VIF
fit2 = lm(y ~ x1 + x2 + x3 + x4 + x5 + x6 + x8 + x9 + x10, dat)

### look at the model estimates
summary(fit2)

#######################################################################
#######################################################################
### On your own:
### What are the VIF of the new full model?



#######################################################################

# What do you think the next step should be?







#######################################################################


## Removing v8 would be a good idea
fit3 = lm(y ~ x1 + x2 + x3 + x4 + x5 + x6 + x9 + x10, dat)
vif(fit3)
summary(fit3)

# Better now.

#######################################################################
#######################################################################
### On your own:
### Now conduct the variable selection process
### What model did you come up with? Are they the same with all methods?
### Choose your final model and name it final.fit



#######################################################################


#### Now let's do some model evaluation with this final model


### get the residuals and plot them
res <- residuals(final.fit)
plot(res)
boxplot(res)
plot(res ~ dat$y)

#######################################################################
#######################################################################
### On your own:
### Plot the residuals against some of the independent variables



#######################################################################

### plot a qq plot to check for normality

qqnorm(res)
qqline(res)## adds the line
## looks pretty good


### conduct a formal test for normality
shapiro.test(res)
## What do the results mean?




### get the predicted values and plot the residuals against them

pred <- predict(final.fit)
plot(res ~ pred)


#######################################################################
#######################################################################
### On your own:
### Plot the observed vs. the predicted values



#######################################################################


### add a line of unity
abline(a = 0 , b = 1)


## Looks pretty good!
## Let's check the regression of observed on predicted just to be sure
summary(lm(dat$y~pred))
    ## Is the intercept 0?
    ## Is the slope 1?

## add the regression line to the plot
abline(lm(dat$y~pred))



##### Let's look at the prediction error

mspe(dat$y, pred)

### and the CCC
epi.ccc(dat$y, pred)




### Let's predict some new values

### First create a new X data matrix with 10 new values for each input variable
### In real life we would select the values for the new data based on
### the inference we wanted to make. We may be interested in assessing the predicted value
### over a range of one predictor while holding the others constant
### Let's see how y varies when x6 ranges from 20-35 and all others are held constant

newdat = data.frame(Intercept = rep(1, 10),
                    x1 = rep(20, 10), 
                    x2 = rep(-3, 10), 
                    x3 = rep(7.8, 10), 
                    x4 = rep(2.5, 10), 
                    x5 = rep(0.4, 10), 
                    x6 = seq(20, 35, length.out = 10))

pred.new = predict(final.fit, newdata= newdat)

plot(pred.new ~ newdat$x6)



